'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var react = require('react');
var invariant = require('tiny-invariant');
var memoize = require('memoize-one');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var invariant__default = /*#__PURE__*/_interopDefaultLegacy(invariant);
var memoize__default = /*#__PURE__*/_interopDefaultLegacy(memoize);

// eslint-disable-next-line import/prefer-default-export
var maxScrollTop = function maxScrollTop(dom) {
  return dom.scrollHeight - dom.clientHeight;
};

var defaultRunScroll = memoize__default['default'](function (domRef) {
  return function (offset) {
    domRef.current.scrollTop = offset;
  };
});
var useIsomorphicLayoutEffect = typeof window !== 'undefined' ? react.useLayoutEffect : react.useEffect;
var index = (function (domRef) {
  var _ref = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
      _ref$initialScroll = _ref.initialScroll,
      initialScroll = _ref$initialScroll === void 0 ? null : _ref$initialScroll,
      _ref$inaccuracy = _ref.inaccuracy,
      inaccuracy = _ref$inaccuracy === void 0 ? 0 : _ref$inaccuracy,
      _ref$runScroll = _ref.runScroll,
      runScroll = _ref$runScroll === void 0 ? defaultRunScroll(domRef) : _ref$runScroll;

  var wasScrolled = react.useRef(null);
  var isScrolled = react.useCallback(function () {
    return domRef.current === null ? false : Math.ceil(domRef.current.scrollTop) >= maxScrollTop(domRef.current) - inaccuracy;
  }, [inaccuracy]);
  react.useEffect(function () {
    var onScroll = function onScroll() {
      wasScrolled.current = isScrolled();
    };

    domRef.current.addEventListener('scroll', onScroll); // in react 17 the cleanup can happen after the element gets unmounted

    return function () {
      var _domRef$current;

      return (_domRef$current = domRef.current) === null || _domRef$current === void 0 ? void 0 : _domRef$current.removeEventListener('scroll', onScroll);
    };
  }, []);
  var scroll = react.useCallback(function (position) {
    !(domRef.current !== null) ? process.env.NODE_ENV !== "production" ? invariant__default['default'](false, "Trying to scroll to the bottom, but no element was found.\n      Did you call this scrollBottom before the component with this hook finished mounting?") : invariant__default['default'](false) : void 0;
    var offset = Math.min(maxScrollTop(domRef.current), position);
    runScroll(offset);
  }, [runScroll]);
  var scrollBottom = react.useCallback(function () {
    scroll(Infinity);
  }, [scroll]);
  var stayScrolled = react.useCallback(function () {
    if (wasScrolled.current) scrollBottom();
    return wasScrolled.current;
  }, [scrollBottom]);
  useIsomorphicLayoutEffect(function () {
    if (initialScroll !== null) {
      scroll(initialScroll);
    }

    wasScrolled.current = isScrolled();
  }, []);
  return {
    scroll: scroll,
    stayScrolled: stayScrolled,
    scrollBottom: scrollBottom,
    isScrolled: isScrolled
  };
});

exports.default = index;
